from PyQt4.QtGui import *  
from PyQt4.QtCore import *  
from PyQt4.QtWebKit import *  
from lxml import html 
from selenium import webdriver
import itertools
import time
import random
import os
import io

# Script for scraping all contact info from people.nasa.gov
# Stephen Reid McLaughlin
# PhD Student, UT Austin School of Information
# stephen.mclaughlin@utexas.edu

# We're going to search the email field for all valid two-character combinations, followed by the wildcard character '*'.
# If a search returns too many results to display on one page (more than 100), we'll append an additional character in the next round, and so on.
# The process ends when none of our searches return too many results to display on a single page.
# I ran each round below by hand and checked the results before proceeding.

os.chdir(os.path.expanduser('~/Dropbox/DataRefuge/NASA_Directory/raw_pages_email'))

chars="abcdefghijklmnopqrstuvwxyz1234567890.!#$%&'*+-/=?^_`{|}~@"   # All currently permitted characters in email addresses.
chars="abcdefghijklmnopqrstuvwxyz1234567890.'-_@"   # All characters actually used in email addresses in NASA directory. Determined by examining test scrape data and by hand searching using double wildcard syntax, as described at people.nasa.gov: "Use two asterisks to find 'term' anywhere..."

driver = webdriver.PhantomJS()                                     # Initializing headless browser, which we'll repeat for each round below.

def js_to_html(url):     # Function that renders a page, including JavaScript content, and returns its content as static html.
    driver.get(url)      # Loading page in headless browser
    return driver.page_source


#### Round 1

stems = [''.join(i) for i in itertools.product(chars, repeat = 2)]      # all two-character combinations

stems=[combo for combo in stems if ('..' not in combo)&(combo[0]!='.')&('@@' not in combo)]  # filtering out illegal character sequences

driver = webdriver.PhantomJS()                          # Initializing headless browser.

try_more_chars=[]

for stem in stems:
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=%s*&phone="%stem
    page_html=js_to_html(url)
    if "Size limit exceeded." in page_html:             # Indicates search returned more than 100 results. Page is thus truncated, so we add stem to our list for the next round and don't save the page.
        print("Size limit exceeded for "+stem)          # Feedback as the script runs
        try_more_chars.append(stem)                     # Keeping a list of stems to expand in next round.
    else:
        filename=(stem+'.html').replace('/',"_slash_")  # Removing any slashes that might appear in filenames.
        if '''<tr class="even">''' in page_html:        # Tag used in all pages that contain contact info, even a single row. Used to filter out empty pages.
            print(filename)                             # Feedback as the script runs
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)
    time.sleep(0.1*random.random())                     # Random sleep between 0 and 0.1 seconds to avoid overloading server.



#### Round 2

#try_more_chars=['aa', 'ab', 'ad', 'al', 'am', 'ar', 'as', 'au', 'ba', 'be', 'bi', 'bl', 'bo', 'ca', 'cc', 'ce', 'cl', 'co', 'cr', 'cs', 'cu', 'cy', 'da', 'de', 'di', 'dm', 'do', 'dr', 'du', 'dw', 'ed', 'el', 'em', 'er', 'es', 'eu', 'ev', 'fa', 'fe', 'fr', 'ga', 'ge', 'gi', 'gl', 'go', 'gr', 'gu', 'ha', 'he', 'hi', 'ho', 'hu', 'id', 'in', 'ir', 'is', 'ja', 'jb', 'jc', 'jh', 'ji', 'jm', 'jo', 'js', 'ju', 'j.', 'ka', 'ke', 'ki', 'ko', 'kr', 'ku', 'ky', 'le', 'li', 'lo', 'lu', 'ly', 'ma', 'mc', 'me', 'mo', 'ms', 'na', 'ne', 'ni', 'no', 'ol', 'pa', 'pe', 'ph', 'pr', 'ra', 're', 'rh', 'ri', 'ro', 'ru', 'ry', 'r_', 'sa', 'sc', 'se', 'si', 'sm', 'so', 'ss', 'st', 'su', 'ta', 'te', 'th', 'ti', 'to', 'tr', 'ty', 'va', 've', 'vi', 'wa', 'we', 'wi', 'ya', 'yo', 'za']

stems=try_more_chars

longer_combos=[]

for stem in stems:
    for char in chars:
        longer_combos.append(stem+char)

longer_combos=[combo for combo in longer_combos if ('..' not in combo)&(combo[0]!='.')&('@@' not in combo)]

driver = webdriver.PhantomJS()

try_more_chars=[]

for stem in longer_combos:
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=%s*&phone="%stem
    page_html=js_to_html(url)
    if "Size limit exceeded." in page_html:
        print("Size limit exceeded for "+stem)
        try_more_chars.append(stem)
    else:
        filename=(stem+'.html').replace('/',"_slash_")
        if '''<tr class="even">''' in page_html:
            print(filename)
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)
    time.sleep(0.1*random.random())



#### Round 3

#try_more_chars=['aar', 'ada', 'adr', 'ala', 'alb', 'ale', 'ali', 'all', 'ama', 'amy', 'art', 'ash', 'bar', 'ben', 'ber', 'bet', 'bil', 'bob', 'car', 'cat', 'ccp', 'cla', 'cli', 'col', 'con', 'cor', 'cra', 'cur', 'cyn', 'dal', 'dan', 'dar', 'dav', 'dea', 'deb', 'den', 'der', 'dia', 'don', 'dou', 'edw', 'eli', 'emi', 'eri', 'fra', 'fre', 'gab', 'gar', 'geo', 'ger', 'gle', 'gra', 'gre', 'han', 'har', 'hea', 'hen', 'jac', 'jam', 'jan', 'jar', 'jas', 'jay', 'jil', 'jim', 'joa', 'joe', 'joh', 'jon', 'jor', 'jos', 'jsc', 'jua', 'jud', 'jul', 'jus', 'kar', 'kat', 'kei', 'kel', 'ken', 'kev', 'kim', 'kri', 'kyl', 'lee', 'leo', 'les', 'lin', 'lis', 'lor', 'lou', 'lyn', 'man', 'mar', 'mat', 'mel', 'mon', 'msf', 'nan', 'nat', 'nic', 'nor', 'pam', 'par', 'pat', 'pau', 'pet', 'phi', 'rac', 'ran', 'ray', 'reb', 'ren', 'ric', 'rob', 'rod', 'rog', 'ron', 'ros', 'rus', 'rya', 'r_h', 'sam', 'san', 'sar', 'sco', 'sea', 'ssc', 'sta', 'ste', 'sus', 'tam', 'ter', 'the', 'tho', 'tim', 'tod', 'tom', 'ton', 'tra', 'tre', 'val', 'van', 'vic', 'vin', 'wal', 'way', 'wen', 'wil', 'zac']

stems=try_more_chars

longer_combos=[]

for stem in stems:
    for char in chars:
        longer_combos.append(stem+char)

longer_combos=[combo for combo in longer_combos if ('..' not in combo)&(combo[0]!='.')&('@@' not in combo)]

driver = webdriver.PhantomJS()

try_more_chars=[]

for stem in longer_combos:
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=%s*&phone="%stem
    page_html=js_to_html(url)
    if "Size limit exceeded." in page_html:
        print("Size limit exceeded for "+stem)
        try_more_chars.append(stem)
    else:
        filename=(stem+'.html').replace('/',"_slash_")
        if '''<tr class="even">''' in page_html:
            print(filename)
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)
    time.sleep(0.1*random.random())

print(try_more_chars)


#### Round 4

#try_more_chars=['aaro', 'adam', 'alan', 'albe', 'alex', 'alic', 'aman', 'amy.', 'ashl', 'barb', 'benj', 'bill', 'carl', 'caro', 'cath', 'ccpe', 'crai', 'curt', 'cynt', 'dani', 'dan.', 'darr', 'davi', 'dean', 'debo', 'debr', 'deni', 'denn', 'dian', 'dona', 'donn', 'doug', 'edwa', 'eliz', 'emil', 'eric', 'erik', 'fran', 'fred', 'gary', 'geor', 'gera', 'glen', 'greg', 'heat', 'henr', 'jack', 'jaco', 'jame', 'jami', 'jane', 'jani', 'jaso', 'jill', 'jim.', 'joan', 'joel', 'joe.', 'john', 'jona', 'jose', 'josh', 'jsc.', 'jsc-', 'juan', 'juli', 'just', 'kare', 'kath', 'keit', 'kell', 'kenn', 'kevi', 'kimb', 'kim.', 'kris', 'kyle', 'lesl', 'lind', 'lisa', 'lori', 'loui', 'lynn', 'marc', 'marg', 'mari', 'mark', 'mart', 'mary', 'matt', 'meli', 'moni', 'msfc', 'nanc', 'nath', 'nich', 'nico', 'pame', 'patr', 'paul', 'pete', 'phil', 'rach', 'rand', 'raym', 'rebe', 'rich', 'rick', 'robe', 'robi', 'roge', 'rona', 'russ', 'ryan', 'r_ho', 'samu', 'sand', 'sara', 'scot', 'sean', 'ssc-', 'stac', 'step', 'stev', 'susa', 'tamm', 'tere', 'terr', 'thom', 'timo', 'todd', 'tom.', 'tony', 'trac', 'vale', 'vict', 'vinc', 'walt', 'wayn', 'wend', 'will', 'zach']
stems=try_more_chars

longer_combos=[]

for stem in stems:
    for char in chars:
        longer_combos.append(stem+char)

longer_combos=[combo for combo in longer_combos if ('..' not in combo)&(combo[0]!='.')&('@@' not in combo)]

driver = webdriver.PhantomJS()

try_more_chars=[]

for stem in longer_combos:
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=%s*&phone="%stem
    page_html=js_to_html(url)
    if "Size limit exceeded." in page_html:
        print("Size limit exceeded for "+stem)
        try_more_chars.append(stem)
    else:
        filename=(stem+'.html').replace('/',"_slash_")
        if '''<tr class="even">''' in page_html:
            print(filename)
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)
    time.sleep(0.1*random.random())

print(try_more_chars)


#### Round 5

#try_more_chars=['aaron', 'adam.', 'alan.', 'alber', 'alexa', 'alex.', 'amand', 'ashle', 'barba', 'benja', 'carlo', 'carl.', 'carol', 'cathe', 'ccper', 'craig', 'cynth', 'danie', 'darre', 'david', 'debor', 'debra', 'denis', 'denni', 'donal', 'donna', 'donn@', 'dougl', 'edwar', 'eliza', 'emily', 'eric.', 'franc', 'frank', 'gary.', 'georg', 'geral', 'glenn', 'grego', 'greg.', 'heath', 'henry', 'jacob', 'james', 'jason', 'jill.', 'joel.', 'john.', 'jonat', 'josep', 'jose.', 'joshu', 'jsc.b', 'jsc-b', 'juan.', 'julie', 'justi', 'karen', 'kathl', 'kathy', 'keith', 'kelly', 'kenne', 'kevin', 'kimbe', 'krist', 'kyle.', 'lesli', 'linda', 'lisa.', 'lori.', 'marga', 'maria', 'mark.', 'marti', 'mary.', 'matth', 'melis', 'msfc-', 'nancy', 'natha', 'nicho', 'nicol', 'pamel', 'patri', 'paul.', 'peter', 'phili', 'phill', 'randa', 'randy', 'raymo', 'rebec', 'richa', 'rober', 'robin', 'roger', 'ronal', 'russe', 'ryan.', 'r_hop', 'samue', 'sandr', 'sarah', 'scott', 'sean.', 'ssc-b', 'steph', 'steve', 'susan', 'teres', 'terry', 'thoma', 'timot', 'todd.', 'victo', 'vince', 'walte', 'wayne', 'willi', 'zacha']

stems=try_more_chars

longer_combos=[]

for stem in stems:
    for char in chars:
        longer_combos.append(stem+char)

longer_combos=[combo for combo in longer_combos if ('..' not in combo)&(combo[0]!='.')&('@@' not in combo)]

driver = webdriver.PhantomJS()

try_more_chars=[]

for stem in longer_combos:
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=%s*&phone="%stem
    page_html=js_to_html(url)
    if "Size limit exceeded." in page_html:
        print("Size limit exceeded for "+stem)
        try_more_chars.append(stem)
    else:
        filename=(stem+'.html').replace('/',"_slash_")
        if '''<tr class="even">''' in page_html:
            print(filename)
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)
    time.sleep(0.1*random.random())

print(try_more_chars)

#### Round 6

#try_more_chars=['aaron.', 'albert', 'alexan', 'amanda', 'barbar', 'benjam', 'carlos', 'carol.', 'cather', 'ccperf', 'craig.', 'cynthi', 'daniel', 'david.', 'debora', 'debra.', 'denise', 'dennis', 'donald', 'donna.', 'donn@t', 'dougla', 'edward', 'elizab', 'emily.', 'franci', 'frank.', 'george', 'gerald', 'glenn.', 'gregor', 'heathe', 'jacob.', 'james.', 'jason.', 'john.c', 'john.d', 'john.m', 'john.r', 'john.w', 'jonath', 'joseph', 'joshua', 'jsc.bl', 'jsc-bl', 'julie.', 'justin', 'karen.', 'kathle', 'keith.', 'kelly.', 'kennet', 'kevin.', 'kimber', 'kristi', 'leslie', 'linda.', 'margar', 'maria.', 'mark.a', 'martin', 'matthe', 'meliss', 'msfc-i', 'nancy.', 'nathan', 'nichol', 'nicole', 'pamela', 'patric', 'peter.', 'philip', 'philli', 'randal', 'randy.', 'raymon', 'rebecc', 'richar', 'robert', 'roger.', 'ronald', 'russel', 'r_hops', 'samuel', 'sandra', 'sarah.', 'scott.', 'ssc-bl', 'stepha', 'stephe', 'steven', 'steve.', 'susan.', 'teresa', 'terry.', 'thomas', 'timoth', 'victor', 'vincen', 'walter', 'wayne.', 'willia', 'zachar']

stems=try_more_chars

longer_combos=[]

for stem in stems:
    for char in chars:
        longer_combos.append(stem+char)

longer_combos=[combo for combo in longer_combos if ('..' not in combo)&(combo[0]!='.')&('@@' not in combo)]

driver = webdriver.PhantomJS()

try_more_chars=[]

for stem in longer_combos:
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=%s*&phone="%stem
    page_html=js_to_html(url)
    if "Size limit exceeded." in page_html:
        print("Size limit exceeded for "+stem)
        try_more_chars.append(stem)
    else:
        filename=(stem+'.html').replace('/',"_slash_")
        if '''<tr class="even">''' in page_html:
            print(filename)
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)
    time.sleep(0.1*random.random())

print(try_more_chars)

#### Round 7

#try_more_chars=['alexand', 'amanda.', 'barbara', 'benjami', 'carlos.', 'catheri', 'ccperf2', 'ccperf3', 'ccperf4', 'cynthia', 'daniel.', 'david.a', 'david.c', 'david.j', 'david.l', 'david.m', 'david.r', 'david.s', 'david.w', 'deborah', 'dennis.', 'donald.', 'donn@to', 'douglas', 'edward.', 'elizabe', 'george.', 'gerald.', 'gregory', 'heather', 'james.a', 'james.e', 'james.m', 'james.r', 'john.m.', 'john.w.', 'jonatha', 'joseph.', 'joshua.', 'jsc.bla', 'jsc-bla', 'justin.', 'kathlee', 'kenneth', 'kimberl', 'kristin', 'leslie.', 'mark.a.', 'martin.', 'matthew', 'melissa', 'msfc-ie', 'nathan.', 'nichola', 'pamela.', 'patrici', 'patrick', 'philip.', 'phillip', 'randall', 'raymond', 'rebecca', 'richard', 'robert.', 'ronald.', 'russell', 'r_hopso', 'samuel.', 'sandra.', 'ssc-bla', 'stephan', 'stephen', 'steven.', 'teresa.', 'thomas.', 'timothy', 'victor.', 'vincent', 'walter.', 'william', 'zachary']

stems=try_more_chars

longer_combos=[]

for stem in stems:
    for char in chars:
        longer_combos.append(stem+char)

longer_combos=[combo for combo in longer_combos if ('..' not in combo)&(combo[0]!='.')&('@@' not in combo)]

driver = webdriver.PhantomJS()

try_more_chars=[]

for stem in longer_combos:
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=%s*&phone="%stem
    page_html=js_to_html(url)
    if "Size limit exceeded." in page_html:
        print("Size limit exceeded for "+stem)
        try_more_chars.append(stem)
    else:
        filename=(stem+'.html').replace('/',"_slash_")
        if '''<tr class="even">''' in page_html:
            print(filename)
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)
    time.sleep(0.1*random.random())

print(try_more_chars)

#### Round 8

#try_more_chars=['alexande', 'barbara.', 'benjamin', 'catherin', 'cynthia.', 'daniel.j', 'daniel.p', 'david.a.', 'david.j.', 'david.l.', 'david.m.', 'deborah.', 'donn@toa', 'douglas.', 'elizabet', 'gregory.', 'heather.', 'james.a.', 'james.e.', 'james.m.', 'james.r.', 'jonathan', 'jsc.blac', 'jsc-blac', 'kathleen', 'kenneth.', 'kimberly', 'matthew.', 'melissa.', 'msfc-iem', 'nicholas', 'patricia', 'patrick.', 'phillip.', 'randall.', 'raymond.', 'rebecca.', 'richard.', 'robert.a', 'robert.c', 'robert.j', 'robert.l', 'robert.m', 'robert.w', 'russell.', 'r_hopson', 'ssc-blac', 'stephani', 'stephen.', 'timothy.', 'william.', 'zachary.']

stems=try_more_chars

longer_combos=[]

for stem in stems:
    for char in chars:
        longer_combos.append(stem+char)

longer_combos=[combo for combo in longer_combos if ('..' not in combo)&(combo[0]!='.')&('@@' not in combo)]

driver = webdriver.PhantomJS()

try_more_chars=[]

for stem in longer_combos:
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=%s*&phone="%stem
    page_html=js_to_html(url)
    if "Size limit exceeded." in page_html:
        print("Size limit exceeded for "+stem)
        try_more_chars.append(stem)
    else:
        filename=(stem+'.html').replace('/',"_slash_")
        if '''<tr class="even">''' in page_html:
            print(filename)
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)
    time.sleep(0.1*random.random())

print(try_more_chars)

#### Round 9

#try_more_chars=['alexander', 'benjamin.', 'catherine', 'donn@toan', 'elizabeth', 'jonathan.', 'jsc.black', 'jsc-black', 'kathleen.', 'kimberly.', 'msfc-iemp', 'nicholas.', 'patricia.', 'richard.a', 'robert.a.', 'robert.j.', 'r_hopson@', 'ssc-black', 'stephanie']

stems=try_more_chars

longer_combos=[]

for stem in stems:
    for char in chars:
        longer_combos.append(stem+char)

longer_combos=[combo for combo in longer_combos if ('..' not in combo)&(combo[0]!='.')&('@@' not in combo)]

driver = webdriver.PhantomJS()

try_more_chars=[]

for stem in longer_combos:
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=%s*&phone="%stem
    page_html=js_to_html(url)
    if "Size limit exceeded." in page_html:
        print("Size limit exceeded for "+stem)
        try_more_chars.append(stem)
    else:
        filename=(stem+'.html').replace('/',"_slash_")
        if '''<tr class="even">''' in page_html:
            print(filename)
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)
    time.sleep(0.1*random.random())

print(try_more_chars)

#### Round 10

#try_more_chars=['alexander.', 'donn@toano', 'elizabeth.', 'jsc.blackh', 'jsc-blackh', 'r_hopson@a', 'ssc-black-', 'stephanie.']

#stems=try_more_chars

stems=['alexander.', 'elizabeth.', 'stephanie.']   # removed stems from >100-times repeated email addresses by hand

longer_combos=[]

for stem in stems:
    for char in chars:
        longer_combos.append(stem+char)

longer_combos=[combo for combo in longer_combos if ('..' not in combo)&(combo[0]!='.')&('@@' not in combo)]

driver = webdriver.PhantomJS()

try_more_chars=[]

for stem in longer_combos:
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=%s*&phone="%stem
    page_html=js_to_html(url)
    if "Size limit exceeded." in page_html:
        print("Size limit exceeded for "+stem)
        try_more_chars.append(stem)
    else:
        filename=(stem+'.html').replace('/',"_slash_")
        if '''<tr class="even">''' in page_html:
            print(filename)
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)
    time.sleep(0.1*random.random())

print(try_more_chars)

## The list try_more_chars comes back empty, so we're done with these stems.


#### Final Round: email addresses repeated >100 times: black hole accounts + r_hopson@aol.com + donn@toanocontractors.com

# Searching for email address stems, along with a single character followed by '*' in last name field.

stems=['donn@toano', 'jsc.blackh', 'jsc-blackh', 'r_hopson@a', 'ssc-black-']

last_name_chars=[item for item in chars if ('*' != item)&('+' != item)&('#' != item)&('&' !=item)]

driver = webdriver.PhantomJS()

try_more_chars=[]

for stem in stems:
    for char in last_name_chars:
        url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=%s*&email=%s*&phone="%(char,stem) #edited
        page_html=js_to_html(url)
        if "Size limit exceeded." in page_html:
            print("Size limit exceeded for "+char)
            try_more_chars.append(char)
        else:
            filename=(stem+"__"+char+'.html').replace('/',"_slash_")
            if '''<tr class="even">''' in page_html:
                print(filename)
                with io.open(filename, 'w', encoding='utf-8') as fo:
                    fo.write(page_html)
    time.sleep(0.1*random.random())

print(try_more_chars)

## The list try_more_chars comes back empty, so we're done.
