from PyQt4.QtGui import *  
from PyQt4.QtCore import *  
from PyQt4.QtWebKit import *  
from lxml import html 
import lxml
from selenium import webdriver
import itertools
import time
import random
import os
import io

# Script for scraping all contact info from people.nasa.gov
# Stephen Reid McLaughlin
# PhD Student, UT Austin School of Information
# stephen.mclaughlin@utexas.edu


os.chdir(os.path.expanduser('~/Dropbox/DataRefuge/NASA_Directory/raw_pages_phone'))

def js_to_html(url,driver):     # Function that renders a page, including JavaScript content, and returns its content as static html.
    driver.get(url)      # Loads page in headless browser
    return driver.page_source


#### Scraping with method used for email addresses and names: starting with one digit, noting which searches return >100 results, and iteratively adding more digits.


try_more_digits=[None]
digits="0123456789"
stems="0123456789"
longer_combos="0123456789"

round=1

while len(try_more_digits)>0:
    driver = webdriver.PhantomJS()
    try_more_digits=[]
    for stem in longer_combos:
        url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=&phone=%s*"%stem
        page_html=js_to_html(url,driver)
        if "Size limit exceeded." in page_html:
            print("Size limit exceeded for "+stem)
            try_more_digits.append(stem)
        else:
            if '''<tr class="even">''' in page_html:
                filename=(stem+'.html')
                print(filename)
                with io.open(filename, 'w', encoding='utf-8') as fo:
                    fo.write(page_html)
    longer_combos=[]
    print(try_more_digits)
    stems=try_more_digits
    for stem in stems:
        for num in digits:
            longer_combos.append(stem+num)
        time.sleep(0.1*random.random())
    print("\nRound "+str(round))
    round+=1



## NASA's directory interface handles phone searches somewhat unpredictably, so we'll try a few more approaches to make sure we've covered our bases:


#### Running brute scrape with double wildcards around 4-digit numbers


driver = webdriver.PhantomJS()

size_exceeded=[]

for i in range(100,10000):
    i_str="%04d"%i
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=&phone=*%s*"%i_str
    page_html=js_to_html(url,driver)
    if "Size limit exceeded." in page_html:
        print("Size limit exceeded for "+i_str)
        size_exceeded.append(i_str)
    if '''<tr class="even">''' in page_html:
        filename=('brute_4d_'+i_str+'.html')
        with io.open(filename, 'w', encoding='utf-8') as fo:
            fo.write(page_html)

# The result:
#size_exceeded=['0749', '1244', '1483', '1792', '2812', '2814', '2817', '4000', '4955', '5555', '7495', '8124', '8148', '8179', '9802']

#### For 4-digit numbers above which returned too many results for one page, we search again with the following formats: X*XXXX* and *XXXX*X


size_exceeded_4d = size_exceeded
size_exceeded=[]
digits="0123456789"

driver = webdriver.PhantomJS()

for i_str in size_exceeded_4d:
    for digit in digits:
        url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=&phone=%s*%s*"%(digit,i_str)
        page_html=js_to_html(url,driver)
        if "Size limit exceeded." in page_html:
            print("Size limit exceeded for "+i_str)
            size_exceeded.append(i_str)
        if '''<tr class="even">''' in page_html:
            filename=('brute_'+digit+'_'+i_str+'.html')
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)

for i_str in size_exceeded_4d:
    for digit in digits:
        url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=&phone=%s*%s*"%(i_str,digit)
        page_html=js_to_html(url,driver)
        if "Size limit exceeded." in page_html:
            print("Size limit exceeded for "+i_str)
            size_exceeded.append(i_str)
        if '''<tr class="even">''' in page_html:
            filename=('brute_'+i_str+'_'+digit+'.html')
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)


#### Now in combination with last name initials:

#size_exceeded=['1244', '1483', '1792', '2812', '2814', '2817', '4000', '4955', '8124', '8148', '8179', '9802']

size_exceeded_4d=sorted(list(set(size_exceeded)))
size_exceeded=[]

name_chars="abcdefghijklmnopqrstuvwxyz',-2"

for i_str in size_exceeded_4d:
    for char in name_chars:
        url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=%s*&email=&phone=*%s*"%(char,i_str)
        page_html=js_to_html(url,driver)
        if "Size limit exceeded." in page_html:
            print("Size limit exceeded for "+i_str)
            size_exceeded.append(i_str)
        if '''<tr class="even">''' in page_html:
            filename=('brute_'+i_str+'_'+char+'.html')
            with io.open(filename, 'w', encoding='utf-8') as fo:
                fo.write(page_html)


# Success! No searches with more than 100 results.



#### Non-numeric terms which appear in phone number field (extracted from phone column in records scraped using email address field)


driver = webdriver.PhantomJS()

size_exceeded=[]

terms=['N/A','No Phone','Phone','TBD','Unknown','VPhon','na','ext','-..']

for term in terms:
    url="https://people.nasa.gov/people/search?firstName=&middleInitial=&lastName=&email=&phone=*%s*"%term
    page_html=js_to_html(url,driver)
    if "Size limit exceeded." in page_html:
        print("Size limit exceeded for "+i_str)
        size_exceeded.append(i_str)
    if '''<tr class="even">''' in page_html:
        filename=('terms_'+term+'.html').replace('/','_slash_')
        with io.open(filename, 'w', encoding='utf-8') as fo:
            fo.write(page_html)

# No searches with more than 100 results, so we're done.


