import unicodecsv
import io
import lxml.html
import os

# Script for parsing scraped search results from people.nasa.gov
# Stephen Reid McLaughlin
# PhD Student, UT Austin School of Information
# stephen.mclaughlin@utexas.edu


master_table=[]                                     # List that will contain all rows extracted from html search results (i.e., a list of lists)


def batch_extract(dir_path):                        # Takes a directory path, parses all html files, and appends rows to master_table
    os.chdir(dir_path)
    for pathname in [item for item in os.listdir(dir_path) if '.html' in item]:
        try:
            html = io.open(pathname).read()
        except:
            print("ERROR: "+pathname)               # Prints error note and breaks loop if a problem occurs while reading a file
            break
        list_of_lists = parse(html)
        for row in list_of_lists:
            master_table.append(row)


def parse(html):                                    # Takes page html, extracts all rows, and returns their contents as a list of lists.
    markup = lxml.html.fromstring(html)
    rows = markup.cssselect("tr")
    list_of_lists = []
    for row in rows:
        temp_row = []
        for td in row.cssselect("td"):
            temp_row.append(unicode(td.text_content().strip()))
        list_of_lists.append(temp_row)
    while [] in list_of_lists:                      # Removing empty rows
        list_of_lists.remove([])
    return list_of_lists



dir_paths=['~/Dropbox/DataRefuge/NASA_Directory/raw_pages_email',\
           '~/Dropbox/DataRefuge/NASA_Directory/raw_pages_last_name',\
           '~/Dropbox/DataRefuge/NASA_Directory/raw_pages_first_name',\
           '~/Dropbox/DataRefuge/NASA_Directory/raw_pages_phone']   # List of relative paths to directories containing scraped html search results

dir_paths=[os.path.expanduser(path) for path in dir_paths]  # Expanding relative paths


for dir_path in dir_paths:                          # Extracting rows from all html pages in list of directory paths
    batch_extract(dir_path)


master_table = list(set(tuple(row) for row in master_table))  # Removing duplicate rows

master_table.sort(key=lambda row: row[0])               # Sorting table by name column

clean_master_table=[]

clean_master_table.append(['Name','Email','Phone']) # Header row

for row in master_table:
    if len(row)==3:                                 # Filtering out any malformed rows; there shouldn't be any.
        clean_master_table.append(list(row))
    else:
        print row                                   # Feedback to see if any filtered rows contain contact info; they shouldn't.


os.chdir(os.path.expanduser("~/Dropbox/DataRefuge/NASA_Directory/"))

with open('NASA_Directory.csv', 'wb') as fo:           #Writing Unicode CSV file
    writer = unicodecsv.writer(fo)
    writer.writerows(clean_master_table)





